unit kErrors;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, savepos, kComp, KParam;

const tab = #09;

type
  TErrorHeader = function:string;

  TCommError = (ceNone,ceCommError,ceCommDebug,ceModbusError,ceModbusDebug,
                ceLineError,ceLineDebug,
                ceUserChange,ceParamChange,cePressDebug,
                ceLast);

  TErrorRec = record
    sc : string;
    expl : string;
    err : boolean;
    show : boolean;
    log : boolean;
  end;

  TErrors = class(TForm)
    FErrorScreen: TMemo;
    PanelSettings: TPanel;
    SavePos1: TSavePos;
    CheckBoxErr: TCheckBox;
    ButtonClear: TButton;
    ButtonOptions: TButton;
    ParamErrorLimit: TkParam;
    ButtonMark: TButton;
    ButtonClose: TButton;
    procedure FormHide(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure CheckBoxErrClick(Sender: TObject);
    procedure ButtonClearClick(Sender: TObject);
    procedure ButtonOptionsClick(Sender: TObject);
    procedure ButtonMarkClick(Sender: TObject);
    procedure ButtonCloseClick(Sender: TObject);
  private
    f:Text;
    NoLogSave : boolean;
    FileName : string;
    LoggedErrors : integer;
    lasttext : string;
  public
    ErrHeader : TErrorHeader;
    ShowErr : boolean;
    constructor Create(AOwner:TComponent); override;
    procedure Add(etype:TCommError;const s:string);
    procedure ShowErrors;
    function ShowComError : boolean;
    function ShowComDebug : boolean;
    function ShowModbusError : boolean;
    function ShowModbusDebug : boolean;
    procedure AddMark;
  published
    property ErrorScreen : TMemo read FErrorScreen write FErrorScreen;
  end;

var
  Errors: TErrors;

  ErrorRec : array[ceNone..ceLast] of TErrorRec = (
    ( sc : 'ne'; expl : 'No error'                                            ),
    ( sc : 'ce'; expl : 'Communication error' ; err : True                    ),
    ( sc : 'cd'; expl : 'Communication debug'                                 ),
    ( sc : 'me'; expl : 'Modbus error'        ; err : True                    ),
    ( sc : 'md'; expl : 'Modbus debug'                                        ),
    ( sc : 'le'; expl : 'Line error'          ; err : True                    ),
    ( sc : 'ld'; expl : 'Line debug'                                          ),
    ( sc : 'uc'; expl : 'User change'                                         ),
    ( sc : 'pc'; expl : 'Param change'                                        ),
    ( sc : 'pr'; expl : 'Press debug'                                         ),
    ( sc : 'ma'; expl : 'An mark'           ; err : False; show:true; log:true)
  );

function NowToStr:string;

implementation

uses erroropt;

{$R *.DFM}

function TR(s:string):string; begin Result := s; end;

function NowToStr:string;
var Hour,Min,Sec,MSec:Word; s:string;
begin
  DecodeTime(Time,Hour,Min,Sec,MSec);
  MSec := MSec div 10;
  s := IntToStr(MSec); if ( Length(s) < 2 ) then s := '0'+s; 
  Result := FormatDateTime('yymmdd hhnnss',now)+'.'+s;
end;

function TErrors.ShowComError : boolean;
begin
  Result := ( ErrorRec[ceCommError].show or ErrorRec[ceCommError].log );
end;

function TErrors.ShowComDebug : boolean;
begin
  Result := ( ErrorRec[ceCommDebug].show or ErrorRec[ceCommDebug].log );
end;

function TErrors.ShowModbusError : boolean;
begin
  Result := ( ErrorRec[ceModbusError].show or ErrorRec[ceModbusError].log );
end;

function TErrors.ShowModbusDebug : boolean;
begin
  Result := ( ErrorRec[ceModbusDebug].show or ErrorRec[ceModbusDebug].log );
end;


constructor TErrors.Create(AOwner:TComponent);
begin
  FileName := 'errorlog.txt';
  AssignFile(F, FileName);
  NoLogSave := false;
  if ( Not FileExists(FileName) ) then begin
    try
      Rewrite(f);
      CloseFile(f);
    except
      NoLogSave := True;
    end;
  end;

  ShowErr := True;
  ErrHeader := NowToStr;
  inherited Create(AOwner);
end;

procedure TErrors.FormHide(Sender: TObject);
begin
  ShowErr := false;
  LoggedErrors := 0;
end;

procedure TErrors.Add(etype:TCommError;const s:string);
var h:string;
begin
  if ( ErrorRec[etype].err ) then begin
    inc(LoggedErrors);
    if ( LoggedErrors >= ParamErrorLimit.Value ) then begin
      LoggedErrors := 0;
      Show;
    end;
  end;

  if not (( ShowErr and ErrorRec[etype].show ) or ( ErrorRec[etype].log )) then exit;
  if ( s = lasttext ) then exit;
  lasttext := s;
  h := ErrorRec[etype].sc+tab+s;
  if ( assigned(ErrHeader) ) then h := ErrHeader+tab+h;
  if ( ShowErr and ErrorRec[etype].show ) then ErrorScreen.Lines.Add(h);
  if ( ( not NoLogSave ) and ErrorRec[etype].log ) then begin
  try
    Append(F);
    Writeln(f,h);
    CloseFile(F);
    NoLogSave := False;
  except
    if ( not NoLogSave ) then
      ShowMessage(TR('Can''t write log file ')+FileName+'!');
    NoLogSave := True;
  end;

  end;
end;

procedure TErrors.FormShow(Sender: TObject);
begin
  ShowErr := CheckBoxErr.Checked;
  LoggedErrors := 0;
end;

procedure TErrors.ShowErrors;
begin
  Show;
end;


procedure TErrors.CheckBoxErrClick(Sender: TObject);
begin
  ShowErr := CheckBoxErr.Checked;
end;

procedure TErrors.ButtonClearClick(Sender: TObject);
begin
  ErrorScreen.Lines.Clear;
  LoggedErrors := 0;
  lasttext := '';
end;

procedure TErrors.ButtonOptionsClick(Sender: TObject);
begin
  FormErrorOpt.Show;
end;

procedure TErrors.AddMark;
begin
  Add(ceLast,'Mark: ----------------------------');
end;

procedure TErrors.ButtonMarkClick(Sender: TObject);
begin
  AddMark;
end;

procedure TErrors.ButtonCloseClick(Sender: TObject);
begin
  Close;
end;

end.
