unit QCCom32;
(*
(c) 1996 Quantum Composers, Bozeman, MT
This file contains Delphi source code to a Delphi Control used for
serial port communications. 32-bit version.
Author: Scott Pinkham
Date Created: 2/9/96
*)
interface

uses
  Windows,
  Classes,
  ExtCtrls,
  SysUtils,
  Forms,
  Dialogs,
  Graphics,
  Controls,
  Buttons,
  StdCtrls;

{ Class Definition ------------------------------}


{ ComPort class }

type
  TCom485 = class(TComponent)

  private  { Private declarations }
    hPort: LongInt; {handle from OpenComm }
    nPort: Integer; { port #, 1-based }
    lBaud: LongInt; { baud rate }
    sInTerminator: ShortString; { response terminator string }
    boolExpired: Boolean; { set true when a timeout occurs }
    boolShowErrors: Boolean; { true to enable error boxes }
    boolInUse: Boolean; { true while port is blocked, waiting for response }

    { Events (method pointers) user can assign code to these }
    pmOnTimeout: TNotifyEvent;	{ timeout event }

    { functions }
    procedure SetTimeout(lTimeout: LongInt);
    function GetTimeout: LongInt;
    procedure SetBaud(lBaudToSet: LongInt);
    procedure SetPort(nPortToSet: Integer);
    function GetInCount: LongInt;
    function IsOpen: Boolean;


  protected

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function Open: Boolean;
    function Write(const sData: ShortString): Boolean;
    function Read: ShortString;
    function ReadThis(const s:ShortString): ShortString;
    function ReadOne: ShortString;
    procedure Flush;
    procedure Close;

  published { Published declarations }
    { timeout for waiting for a response, in mS }
    property ResponseTime: LongInt read GetTimeout write SetTimeout;
    { timeout event }
    property OnTimeout: TNotifyEvent read pmOnTimeout write pmOnTimeout;
    { flag set when a timeout occured }
    property TimedOut: Boolean read boolExpired;
    { baud rate }
    property Baud: LongInt read lBaud write SetBaud;
    { COM port }
    property Port: Integer read nPort write SetPort;
    { number of characters received }
    property InCount: LongInt read GetInCount;
    { string to signal end of input from external device }
    property EndOfResponse: ShortString read sInTerminator write sInTerminator;
    { flag to enable showing error message boxes }
    property ShowErrors: Boolean read boolShowErrors write boolShowErrors;
    { flag indicating if port is open }
    property Opened: Boolean read IsOpen;
    { flag indication if port is blocked }
    property InUse: Boolean read boolInUse;
end;

procedure Register;

implementation


{
Register the component with the Delphi IDE
}
procedure Register;
begin
	RegisterComponents('QC', [TCom485]);
end;

{
Component constructor
}
constructor TCom485.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  { set default property values }
  hPort := INVALID_HANDLE_VALUE; { invalidate to start }
  lBaud := 9600;
  nPort := 1;
  boolExpired := False;
  boolShowErrors := True;
  boolInUse := False;
  pmOnTimeout := NIL;

end;

{ Component destructor }
destructor TCom485.Destroy;
begin
  Close;                { close the com port (if open) }
  inherited Destroy;	{ destroy ancestor class }
end;


procedure TCom485.SetTimeout(lTimeout: LongInt);
begin
{ Tekemtt }
end;

{ Get the current timeout setting }
function TCom485.GetTimeout: LongInt;
begin
  Result := 0;
end;

{ Return True if port is open }
function TCom485.IsOpen: Boolean;
begin
  //if hPort <> INVALID_HANDLE_VALUE then Result := True else Result := False;
  Result := (hPort <> INVALID_HANDLE_VALUE);
end;

{ Set the baud rate property }
procedure TCom485.SetBaud(lBaudToSet: LongInt);
begin
  if lBaudToSet = lBaud then exit;
  lBaud := lBaudToSet;
  if IsOpen then begin Close; Open; end;
end;

{ Set the Port property }
procedure TCom485.SetPort(nPortToSet: Integer);
begin
  if nPortToSet = nPort then exit;
  nPort := nPortToSet;
  if IsOpen then begin Close; Open; end;
end;

{ Opens the COM port, returns True if ok }
function TCom485.Open: Boolean;
var
	sCom: String;
	dcbPort: TDCB; {device control block }
	boolAbort: Boolean;
	sErrMsg: ShortString;
        tmouts : TCommTimeouts;
begin
   { init }
   Result := False;
   boolAbort := True;
   //boolInUse := True;

   if IsOpen then Close;

   { try to open the port }
   repeat
     sCom := 'COM' + IntToStr(nPort);
     hPort := CreateFile(PChar(sCom), GENERIC_READ or GENERIC_WRITE, 0, nil,
                             OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, LongInt(0));

     if ( Not IsOpen ) and boolShowErrors then begin
       if MessageDlg('Error opening COM' + IntToStr(nPort) + ': ' + sErrMsg,
           mtWarning, [mbAbort, mbRetry], 0) = idAbort then
         boolAbort := True
       else
         boolAbort := False;
       end;
   until ( IsOpen ) or (boolAbort = True);

   { set the baud rate and other parameters }
   if ( Not IsOpen ) then exit;
   if ( Not GetCommState(hPort, dcbPort) ) then exit;
   { fill in the fields of the structure }
   dcbPort.BaudRate := lBaud;
   dcbPort.ByteSize := 8;
   dcbPort.Parity := NOPARITY;
   dcbPort.StopBits := ONESTOPBIT;
   dcbPort.Flags := 0;
   { flag bit fields:
   dcb_Binary, dcb_Parity, dcb_OutxCtsFlow, dcb_fOutxDsrFlow,
   dcb_fOutX, dcb_fInX, dcb_DtrFlow, dcb_RtsFlow
   }
   SetCommState(hPort, dcbPort);

   GetCommTimeouts(hPort,tmouts);
    tmouts.ReadIntervalTimeout := 4;

    tmouts.ReadTotalTimeoutMultiplier := 4;

    tmouts.ReadTotalTimeoutConstant := 500;
{
    tmouts.WriteTotalTimeoutMultiplier;
    tmouts.WriteTotalTimeoutConstant;
}
   SetCommTimeouts(hPort,tmouts);
   GetCommTimeouts(hPort,tmouts);
   { return True if handle is valid }
   Result := IsOpen;
   //boolInUse := False;

end;

{ Close the COM port }
procedure TCom485.Close;
begin
  if IsOpen then CloseHandle(hPort);
  hPort := INVALID_HANDLE_VALUE;
  //boolInUse := False;
end;

function CChar(const s:ShortString):PChar;
begin
  Result := @s[1];
end;

{ Write a string out the COM port, return true if all chars written }
function TCom485.Write(const sData: ShortString): Boolean;
var
  dwCharsWritten: DWord;
  event : integer;
begin
  { init }
  //boolInUse := True;
  dwCharsWritten := 0;
  Result := False; { default to error return }

  if Not IsOpen then exit;
  EscapeCommFunction(hPort,SETDTR);
  WriteFile(hPort, sData[1], Length(sData), dwCharsWritten, nil);
  if dwCharsWritten = Length(sData) then Result := True;
 // Sleep(1);
//  SetCommMask(hPort,EV_TXEMPTY);
//  WaitCommEvent(hPort,event,NIL);
  ReadThis(sData);
  EscapeCommFunction(hPort,CLRDTR);

  //boolInUse := False;

end;

{ Return the number of bytes waiting in the queue }
function TCom485.GetInCount: LongInt;
var
  statPort: TCOMSTAT;
  dwErrorCode: DWord;
begin
  Result := 0;
  if Not IsOpen then exit;

  ClearCommError(hPort, dwErrorCode, @statPort);
  Result := statPort.cbInQue;
end;

{
Reads a string from the port, puts it into pchBuffer, returns the
number of characters read
}
function TCom485.Read: ShortString;
var cbCharsAvailable, cbCharsRead: DWord;
    boolExit: Boolean;
    sBuffer: ShortString;
begin
  { init }
  Result := '';

{ check boolInUse in case of rentrancy }
  if boolInUse then begin
    if boolShowErrors then ShowMessage('Port is in use -- operation aborted');
    Exit;
  end;

  if Not IsOpen then exit;
  boolInUse := True;

{ if no terminator is defined, simply read any available data and return }
  if Length(sInTerminator) = 0 then begin
//      cbCharsAvailable := GetInCount;
    cbCharsAvailable := 200;
    if cbCharsAvailable > 0 then begin
      SetLength(Result, cbCharsAvailable + 1); { allocate space }
      ReadFile(hPort, sBuffer[1], cbCharsAvailable, cbCharsRead, nil);
      SetLength(sBuffer, cbCharsRead); { adjust length }
      Result := sBuffer;
    end;
  end
  else { a terminator is defined, so read port until terminator found or timed out }
  begin
    boolExit := False;
    repeat
      boolExpired := False;

      { loop until timeout or terminator recieved }
      repeat
        cbCharsAvailable := GetInCount;
        if cbCharsAvailable > 0 then begin
          if cbCharsAvailable >= Length(sBuffer) then
                  SetLength(sBuffer, cbCharsAvailable + 1); { allocate space }

          ReadFile(hPort, sBuffer[1], cbCharsAvailable, cbCharsRead, nil);

          { append chars read to end of result buffer }
          Result := Result + Copy(sBuffer, 0, cbCharsRead);
        end;

        if Pos(sInTerminator, Result) <> 0 then begin
          boolExit := True;
        end
        else begin
          { give timer message a chance to occur - watch out for rentrancy!!}
          Application.ProcessMessages;
        end;

      until (boolExpired) or (boolExit);


      { If timed out, then give user a chance to retry }
      if boolExpired and boolShowErrors then
      begin
        if MessageDlg('Timeout waiting for response.', mtWarning,
           [mbAbort, mbRetry], 0) = idAbort then boolExit := True;
      end;

    until boolExit = True;
  end;
  boolInUse := False;
end;

function TCom485.ReadThis(const s:ShortString): ShortString;
var sBuffer: ShortString;
    cbCharsAvailable, cbCharsRead: DWord;
begin
  if boolInUse then begin
    if boolShowErrors then ShowMessage('Port is in use -- operation aborted');
    Exit;
  end;

  if not IsOpen then exit;

  boolInUse := True;

  cbCharsAvailable := Length(s);
  ReadFile(hPort, sBuffer[1], cbCharsAvailable, cbCharsRead, nil);
  SetLength(sBuffer, cbCharsRead); { adjust length }
  Result := sBuffer;

  boolInUse := False;
end;


function TCom485.ReadOne: ShortString;
var cbCharsAvailable, cbCharsRead: DWord;
    boolExit: Boolean;
begin
  { init }
  Result := '';

{ check boolInUse in case of rentrancy }
  if boolInUse then begin
    if boolShowErrors then ShowMessage('Port is in use -- operation aborted');
    Exit;
  end;

  if not IsOpen then exit;

  boolInUse := True;
  cbCharsAvailable := 1;
  ReadFile(hPort, Result[1], cbCharsAvailable, cbCharsRead, nil);
  SetLength(Result, cbCharsRead); { adjust length }
  boolInUse := False;
end;


{ Flush the port by reading any characters in the queue }
procedure TCom485.Flush;
begin
  if Not IsOpen then exit;
  PurgeComm(hPort, PURGE_TXABORT or PURGE_RXABORT or PURGE_TXCLEAR or PURGE_RXCLEAR);
end;


end.
